/********************************************************************
*
* KILLIT.C.....Application Kill Library
*
* Author:      John P. Repko
* Last Update: 03/30/94
* Compiler:    WatCom C, version 9
*
* Description: This library provides two functions to enable you
*              to get all users out of your application within a
*              particular time frame.  It's useful in multi-user
*              applications where you don't have the time or
*              inclination to track down all users and ask them
*              to exit before you make system changes that require
*              exclusive access to files.  Once the "Kill" sequence
*              is initiated, a "countdown" clock appears on the
*              screen showing them how much time they have before
*              they are bumped.
*
*              Since the Library is hooked to the FoxPro Idle
*              procedure it will work regardless of the process the
*              user is performing (e.g.: menus, reads, etc...).
*              FoxPro is kind enough to halt processing of Idle
*              messages during critical file maintenance such as
*              REINDEX, PACK, etc...
*
*              The "Kill" sequence is initiated by simply creating
*              a file with a pre-determined name in a pre-determined
*              directory.
*
*********************************************************************
*
* Function:    KILLACTIVE(killfile,testfreq,countdown,clockloc)
*
* Description: This function activates the kill procedure.  It should
*              be called at the beginning of your program.
*
* Parameters:  killfile  - <char> - Required.  This is the path and
*                                   name of the file that signifies
*                                   the kill sequence to begin.
*              testfreq  - <int > - Required.  The frequency, in
*                                   seconds, that the presence of the
*                                   kill file is looked for.
*              countdown - <int > - Required.  The duration, in
*                                   seconds that the system will
*                                   run after the kill file is
*                                   found.
*              clockloc  - <int > - Required.  The screen row that
*                                   you want the the count down
*                                   clock to appear on.
*              
*********************************************************************
*
* Function:    KILLDEAD()
*
* Description: This function deactivates the kill procedure without
*              unloading the library.  I can't think of why you would
*              want to use it, but it was useful for debugging.
*
* Parameters:  None.
*
*********************************************************************
*
* Sample Use:  *--Put at top of Main Program--*
*              IF FILE(MLAppDir()+"killit.out")
*                QUIT
*              ELSE
*                SET LIBR TO killit ADDITIVE
*                =KillActive(MLAppDir()+"killit.out",10,60,48)
*              ENDIF
*              .
*              .
*              ...rest of program
*              .
*              .
*              PROCEDURE MLAppDir
*              PRIVATE   tmpstr
*              tmpstr = SYS(16,0)
*              RETURN LEFT(tmpstr,RAT("\",tmpstr))
*
*********************************************************************/

/* Include Files */
#include <stdio.h>
#include <pro_ext.h>
#include <time.h>
#include <dos.h>
#include <graph.h>
#include <i86.h>

/* Function Declarations */
void FAR killwindow(void);
void FAR IdleHandler(WHandle wh, EventRec *ev);
void FAR KillActive(ParamBlk FAR *parm);
void FAR KillDead(ParamBlk FAR *parm);
void FAR KillDead2(ParamBlk FAR *parm);
void FAR checkfile(void);
void FAR noisy(int stype);
void FAR NullTerminate(Value FAR *cVal);
void FAR StripParms(ParamBlk FAR *parm);
void FAR CleanKill(void);

/* Internal Variables */
static unsigned IdlerID;
static time_t   old_time=1L;
static char     rundown=FALSE;
static WHANDLE  wh2;
static int      idle_on=FALSE;

/* Pass as Parameters */
static int  testfreq=5;
static int  countdown=10;
static int  clockloc=48;
static char killfile[120]="killit.out";

/* Required PLB Declarations */
FoxInfo myFoxInfo[] =
{
  {"KILLACTIVE", (FPFI) KillActive, 4,            "C,I,I,I"},
  {"KILLDEAD",   (FPFI) KillDead,   0,            ""       },
  {"KILLDEAD2",  (FPFI) KillDead2,  CALLONUNLOAD, ""       }
};
FoxTable _FoxTable =
{
  (FoxTable FAR *) 0, sizeof(myFoxInfo)/sizeof(FoxInfo), myFoxInfo
};



/********************************************************************
*
* FUNCTION:    CleanKill(void)
*
* Type:        Internal Library Routine
*
* Description: Clean up routine for Idler whenever QUIT, SET LIBR TO,
*              or RELEASE LIBR is called.
*
*********************************************************************/
void FAR CleanKill(void)
{
  WHANDLE fwh;
  Point   wp;
  char FAR wt[80];
  
  wp.h = 1;
  wp.v = clockloc;
  
  _FindWindow(&fwh,wp);
  
  _WTitle(fwh,wt);
  
  if (_StrCmp(wt,"  System Shut Down  ")==0)
  
  {
    _WClose(wh2);
    rundown = FALSE;
  }

  if (idle_on)
  {
    idle_on = FALSE;
    _DeActivateIdle(IdlerID);
  }
}



/********************************************************************
*
* FUNCTION:    StripParms(ParamBlk FAR *parm)
*
* Type:        Internal Library Routine
*
* Description: Reassignment of passed parameters to static library
*              variables.
*
*********************************************************************/
void FAR StripParms(ParamBlk FAR *parm)
{
  NullTerminate(&parm->p[0].val);
  _StrCpy(killfile,_HandToPtr(parm->p[0].val.ev_handle));

  old_time  = 1L;
  testfreq  = parm->p[1].val.ev_long ;
  countdown = parm->p[2].val.ev_long ;
  clockloc  = parm->p[3].val.ev_long ;
}



/********************************************************************
*
* FUNCTION:    NullTerminate(Value FAR *cVal)
*
* Type:        Internal Library Routine
*
* Description: Null terminates character strings passed from FoxPro.
*
*********************************************************************/
void FAR NullTerminate(Value FAR *cVal)
{
  if (!_SetHandSize(cVal->ev_handle,cVal->ev_length+1))
  {
    _Error(182);
  }
  ((char FAR *) _HandToPtr(cVal->ev_handle))[cVal->ev_length]='\0';
}



/********************************************************************
*
* FUNCTION:    KillActive(ParamBlk FAR *parm)
*
* Type:        External Library Routine (called from FoxPro)
*
* Description: Activates the test for killing the application.
*
*********************************************************************/
void FAR KillActive(ParamBlk FAR *parm)
{

  if (idle_on)
  {
    KillDead(parm);
  }

  StripParms(parm);
  idle_on = TRUE;
  IdlerID = _ActivateIdle((FPFI) IdleHandler);

}



/********************************************************************
*
* FUNCTION:    noisy(int stype)
*
* Type:        Internal Library Routine
*
* Description: Makes appropriate noises.
*
*********************************************************************/
void FAR noisy(int stype)
{
  int i;

  if (stype)
  {
    sound(1650);
    sleep(1);
    nosound();
  }
  else
  {
    sound(1650);
    for (i=0;i<2000;i++);
    nosound();
  }
}



/********************************************************************
*
* FUNCTION:    checkfile(void)
*
* Type:        Internal Library Routine
*
* Description: Looks for existence of the "Kill" file.
*
*********************************************************************/
void FAR checkfile(void)
{
  struct find_t ffblk;
  int foundit;

  foundit = _dos_findfirst(killfile,_A_NORMAL,&ffblk);

  if (foundit == 0)
  {
    killwindow();  
  }

}



/********************************************************************
*
* FUNCTION:    killwindow(void)
*
* Type:        Internal Library Routine
*
* Description: Notifies user that their time is up. Builds countdown
*              clock window.
*
*********************************************************************/
void FAR killwindow(void)
{
  struct videoconfig vc;
  int rows,cols;
  char customScheme[12] = { WHITE_ON|RED|BRIGHT,
                            RED_ON  |BLACK,
                            BROWN_ON|RED|BRIGHT,
                            CYAN_ON |BLUE,
                            WHITE_ON|RED|BRIGHT,
                            BROWN_ON|BROWN,
                            MAGENTA_ON|MAGENTA,
                            RED_ON  |MAGENTA,
                            BROWN_ON|GREEN,
                            BLACK_ON|CYAN,
                            BLUE_ON |CYAN,
                            BLUE_ON |CYAN,
                          };

  _DeActivateIdle(IdlerID);

  _getvideoconfig(&vc);

  rows = (int) ((vc.numtextrows / 2)-5);
  cols = (int) ((vc.numtextcols / 2)-30);

  wh2 = _WOpen( rows, cols, rows+8, cols+58,
        WMODAL+SHADOW,
        -1,
        customScheme,
        WO_PANELBORDER );

  if (wh2 != 0)
  {
    _WSetTitle( wh2, "  WARNING!  " );
    _WSetFooter( wh2, " Press Any Key to Exit... " );
    _WPutStr(wh2,"\n\n          The System will SHUT DOWN very soon!!");
    _WPutStr(wh2,"\n\n Please Finish your Current Task and leave the System...");
    _WShow( wh2 );

    noisy(TRUE);

    _InKey(90,HIDECURSOR|MOUSEACTIVE);
    _WClose(wh2);



    wh2 = _WOpen( clockloc, 0, clockloc+1, vc.numtextcols-1,
          WMODAL,
          -1,
          customScheme,
          WO_SINGLEBOX );

    if (wh2 != 0)
    {
      _WSetTitle ( wh2, "  System Shut Down  " );
      _WSetFooter( wh2, "  Seconds Remaining  " );
      _WShow( wh2 );
    }

    rundown = TRUE;
    testfreq = 1;
  }

  old_time = time(NULL);
  IdlerID = _ActivateIdle((FPFI) IdleHandler);

}



/********************************************************************
*
* FUNCTION:    IdleHandler(WHandle wh, EventRec *ev)
*
* Type:        Internal Library Routine
*
* Description: This is the Idle procedure that is attached to the
*              FoxPro list of Idle routines.
*
*********************************************************************/
void FAR IdleHandler(WHandle wh, EventRec *ev)
{
  time_t cur_time,diff_time;

  cur_time = time(NULL);

  diff_time = (cur_time - old_time);

  if (diff_time >= testfreq)
  {
    old_time = cur_time;

    if (!rundown)
    {
      checkfile();
    }
    else
    {
      char fstr[40]="  ";
      char nstr[10];

      noisy(FALSE);
      countdown = (countdown - diff_time);

      if (countdown <= 0)
      {
        rundown = FALSE;
        _Execute("QUIT");
      }

      itoa(countdown,nstr,10);
      
      strcat(fstr,nstr);
      strcat(fstr," Seconds Remaining  ");

      _WSetFooter( wh2, fstr );
    }
  }
}



/********************************************************************
*
* FUNCTION:    KillDead(ParamBlk FAR *parm)
*
* Type:        External Library Routine (called from FoxPro)
*
* Description: Can be used to deactivate the kill procedure without
*              unloading the library.
*
*********************************************************************/
void FAR KillDead(ParamBlk FAR *parm)
{
  CleanKill();
}



/********************************************************************
*
* FUNCTION:    KillDead2(ParamBlk FAR *parm)
*
* Type:        Internal Library Routine
*
* Description: This function is automatically called by FoxPro
*              whenever QUIT, SET LIBR TO, or RELEASE LIBRARY killit
*              is processed in the user code.
*
*********************************************************************/
void FAR KillDead2(ParamBlk FAR *parm)
{
  CleanKill();
}



/************************* EOF...KILLIT.C...************************/
